package ru.myitschool.work.ui.searchuser

import android.os.Bundle
import ru.myitschool.work.utils.collectWhenStarted
import ru.myitschool.work.utils.visibleOrGone
import android.view.View
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import dagger.hilt.android.AndroidEntryPoint
import ru.myitschool.work.R
import ru.myitschool.work.databinding.FragmentSearchUserBinding
import ru.myitschool.work.ui.profile.ProfileDestination


@AndroidEntryPoint
class SearchUserFragment : Fragment(R.layout.fragment_search_user) {

    private var _binding: FragmentSearchUserBinding? = null
    private val binding: FragmentSearchUserBinding get() = _binding!!

    private val viewModel: SearchUserViewModel by viewModels()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentSearchUserBinding.bind(view)

        initCallback()
        subscribe()
        // Set the initial state to not show the loading state

    }
    private fun initCallback() {
        binding.floatingActionButton2.setOnClickListener { viewModel.closeSearchUser() }
        binding.login.setOnClickListener {
            // Clear error text before starting search
            binding.error?.text = ""
            // Set loading state before calling search method
            viewModel.setLoadingState()
            viewModel.searchUser(binding.username.text.toString())
        }
    }

    private fun subscribe() {
        viewModel.action.collectWhenStarted(this) { action ->
            when (action) {
                is SearchUserViewModel.Action.OpenProfile -> {
                    findNavController().navigate(ProfileDestination) {
                        popUpTo<ProfileDestination> { inclusive = true }
                    }
                }
            }
        }

        viewModel.state.collectWhenStarted(this) { state ->
            //binding.loading?.visibleOrGone(state is SearchUserViewModel.State.Loading)
            // Manage visibility of progress bar and error text
            binding.error?.visibleOrGone(state is SearchUserViewModel.State.Error)
            //binding.loading?.visibility = View.GONE

            when (state) {
                //TODO ПОчинить прогресс бар
                is SearchUserViewModel.State.Error -> {
                    // Set error text
                    binding.error?.text = state.errorText.localizedMessage
        //            binding.loading?.visibility = View.GONE
                }
                is SearchUserViewModel.State.Loading -> {
                    // Additional logic can be added here if needed
       //             binding.loading?.visibility = View.VISIBLE
                }
            }
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null // Release binding to avoid memory leaks
    }
}